package de.ozgcloud.vorgang.common.migration;

import static org.assertj.core.api.Assertions.*;

import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
class RenameUtilITCase {

	private static final String OLD_CLIENT_NAME = "OldClientName";
	private static final String NEW_CLIENT_NAME = "NewClientName";

	@Autowired
	private MigrationDbTestUtils dbTestUtils;

	@DisplayName("VorgangAttachedItem")
	@Nested
	class TestVorgangAttachedItem {

		private static final Document OLD_CLIENT_NAME_VORGANG_ATTACHED_ITEM = RenameTestFactory.createVorgangAttachedItem(OLD_CLIENT_NAME);
		private static final Document NEW_CLIENT_NAME_VORGANG_ATTACHED_ITEM = RenameTestFactory.createVorgangAttachedItem(NEW_CLIENT_NAME);

		@BeforeEach
		public void init() {
			dbTestUtils.dropVorgangAttachedItemCollection();
		}

		@DisplayName("Create find query")
		@Nested
		class TestCreateFindQuery {

			@Test
			void shouldFindWithOldClientName() {
				dbTestUtils.saveVorgangAttachedItem(OLD_CLIENT_NAME_VORGANG_ATTACHED_ITEM);

				var vorgang = dbTestUtils.findVorgangAttachedItem(RenameUtil.createFindVorgangAttachedItemQuery(OLD_CLIENT_NAME));

				assertThat(vorgang).hasSize(1);
			}

			@Test
			void shouldNOTFindWithOtherClientName() {
				dbTestUtils.saveVorgangAttachedItem(NEW_CLIENT_NAME_VORGANG_ATTACHED_ITEM);

				var vorgang = dbTestUtils.findVorgangAttachedItem(RenameUtil.createFindVorgangAttachedItemQuery(OLD_CLIENT_NAME));

				assertThat(vorgang).isEmpty();
			}
		}
	}

	@DisplayName("GridFsFiles")
	@Nested
	class TestGridFsFiles {

		private static final Document OLD_CLIENT_NAME_GRID_FS_FILE = RenameTestFactory.createGridFsFile(OLD_CLIENT_NAME);
		private static final Document NEW_CLIENT_NAME_GRID_FS_FILE = RenameTestFactory.createGridFsFile(NEW_CLIENT_NAME);

		@BeforeEach
		public void init() {
			dbTestUtils.dropGridFsFilesCollection();
		}

		@DisplayName("Create find query")
		@Nested
		class TestCreateFindQuery {

			@Test
			void shouldFindWithOldClientName() {
				dbTestUtils.saveGridFsFile(OLD_CLIENT_NAME_GRID_FS_FILE);

				var gridFsFiles = dbTestUtils.findGridFsFile(RenameUtil.createMetadataClientMatchingDocument(OLD_CLIENT_NAME));

				assertThat(gridFsFiles).hasSize(1);
			}

			@Test
			void shouldNOTFindWithOtherClientName() {
				dbTestUtils.saveGridFsFile(NEW_CLIENT_NAME_GRID_FS_FILE);

				var gridFsFiles = dbTestUtils.findGridFsFile(RenameUtil.createMetadataClientMatchingDocument(OLD_CLIENT_NAME));

				assertThat(gridFsFiles).isEmpty();
			}
		}
	}

	@DisplayName("Vorgang ClientAttributes")
	@Nested
	class TestVorgangClientAttributes {

		private static final Document OLD_CLIENT_NAME_VORGANG = RenameTestFactory.createVorgangWithClientAttributes(OLD_CLIENT_NAME);
		private static final Document NEW_CLIENT_NAME_VORGANG = RenameTestFactory.createVorgangWithClientAttributes(NEW_CLIENT_NAME);

		@BeforeEach
		public void init() {
			dbTestUtils.dropVorgangCollection();
		}

		@DisplayName("Create find query")
		@Nested
		class TestCreateFindQuery {

			@Test
			void shouldFindWithOldClientName() {
				dbTestUtils.saveVorgang(OLD_CLIENT_NAME_VORGANG);

				var vorgang = dbTestUtils.findVorgang(RenameUtil.createFindVorgangClientAttributesQuery(OLD_CLIENT_NAME));

				assertThat(vorgang).hasSize(1);
			}

			@Test
			void shouldNOTFindWithOtherClientName() {
				dbTestUtils.saveVorgang(NEW_CLIENT_NAME_VORGANG);

				var vorgang = dbTestUtils.findVorgang(RenameUtil.createFindVorgangClientAttributesQuery(OLD_CLIENT_NAME));

				assertThat(vorgang).isEmpty();
			}
		}
	}
}